'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

exports.default = function (program, { hasExports, sources, moduleName }) {
    let prevDecl = null,
        deps = sources.map(source => {
        const localName = source[0].name,
              importName = source[1].value.replace('ym:', ''),
              isPrevDecl = moduleName === importName;

        isPrevDecl && (prevDecl = localName);

        return { localName, importName, isPrevDecl };
    });

    const body = program.node.body,
          ymArgs = [],
          ymBodyArgNames = [];

    if (deps.length) {
        deps = deps.filter(({ isPrevDecl }) => !hasExports || !isPrevDecl).map(d => {
            ymBodyArgNames.push(_babelCore.types.identifier(d.localName));
            return _babelCore.types.stringLiteral(d.importName);
        });

        deps.length && ymArgs.push(_babelCore.types.arrayExpression(deps));
    }

    if (!(deps.length || hasExports)) return;

    if (hasExports) {
        if (!moduleName) throw program.buildCodeFrameError('Module name required');

        ymArgs.unshift(_babelCore.types.stringLiteral(moduleName));
        ymBodyArgNames.unshift(_babelCore.types.identifier('provide'));

        prevDecl && ymBodyArgNames.push(_babelCore.types.identifier(prevDecl));

        program.get('body.0').scope.push(_babelCore.types.variableDeclarator(_babelCore.types.identifier('exports'), _babelCore.types.objectExpression([])));

        program.node.body.push(_babelCore.types.expressionStatement(_babelCore.types.callExpression(_babelCore.types.identifier('provide'), [_babelCore.types.identifier('exports')])));
    }

    ymArgs.push(_babelCore.types.functionExpression(null, ymBodyArgNames, _babelCore.types.blockStatement(program.node.body)));

    var call = _babelCore.types.callExpression(_babelCore.types.memberExpression(_babelCore.types.identifier('modules'), _babelCore.types.identifier(hasExports ? 'define' : 'require'), false), ymArgs);

    program.node.body = [_babelCore.types.expressionStatement(call)];
};

var _babelCore = require('babel-core');